@@ -1,21 +1,8 @@
-/***********************************************************************
- *
- * Copyright (C) 2009, 2012, 2013, 2014, 2015, 2016, 2017, 2018, 2019, 2020 Graeme Gott <graeme@gottcode.org>
- *
- * This program is free software: you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation, either version 3 of the License, or
- * (at your option) any later version.
- *
- * This program is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
- *
- * You should have received a copy of the GNU General Public License
- * along with this program.  If not, see <http://www.gnu.org/licenses/>.
- *
- ***********************************************************************/
+/*
+	SPDX-FileCopyrightText: 2009-2021 Graeme Gott <graeme@gottcode.org>
+
+	SPDX-License-Identifier: GPL-3.0-or-later
+*/
 
 #include "window.h"
 
@@ -25,7 +12,7 @@
 #include "locale_dialog.h"
 #include "new_game_dialog.h"
 #include "pattern.h"
-#include "score_board.h"
+#include "scores_dialog.h"
 #include "view.h"
 
 #include <QApplication>
@@ -41,7 +28,10 @@
 #include <algorithm>
 #include <cmath>
 
-Window::Window() {
+//-----------------------------------------------------------------------------
+
+Window::Window()
+{
 	m_board = new Board(this);
 	connect(m_board, &Board::finished, this, &Window::gameFinished);
 	connect(m_board, &Board::started, this, &Window::gameStarted);
@@ -52,13 +42,11 @@ Window::Window() {
 
 	View* view = new View(m_board, contents);
 
-	m_scores = new ScoreBoard(this);
-
 	m_definitions = new Definitions(m_board->words(), this);
 	connect(m_board, &Board::wordAdded, m_definitions, &Definitions::addWord);
 	connect(m_board, &Board::wordSolved, m_definitions, &Definitions::solveWord);
 	connect(m_board, &Board::wordSelected, m_definitions, &Definitions::selectWord);
-	connect(m_board, &Board::loading, m_definitions, &Definitions::clear);
+	connect(m_board, &Board::loading, m_definitions, &Definitions::clearWords);
 
 	// Create success message
 	m_success = new QLabel(contents);
@@ -91,7 +79,7 @@ Window::Window() {
 	connect(m_board, &Board::loading, m_success, &QLabel::hide);
 
 	// Create overlay background
-	QLabel* overlay = new QLabel(this);
+	m_overlay = new QLabel(this);
 
 	f = font();
 	f.setPixelSize(20);
@@ -99,8 +87,8 @@ Window::Window() {
 	width = std::max(metrics.boundingRect(tr("Loading")).width(), metrics.boundingRect(tr("Paused")).width());
 	for (int i = 0; i < 10; ++i) {
 		QString test(6, QChar(i + 48));
-		test.insert(4, QLatin1Char(':'));
-		test.insert(2, QLatin1Char(':'));
+		test.insert(4, QChar(':'));
+		test.insert(2, QChar(':'));
 		width = std::max(width, metrics.boundingRect(test).width());
 	}
 	pixmap = QPixmap(QSize(width + 82, 32) * ratio);
@@ -114,16 +102,19 @@ Window::Window() {
 		painter.setRenderHint(QPainter::Antialiasing, true);
 		painter.drawRoundedRect(0, -32, width + 82, 64, 5, 5);
 	}
-	overlay->setPixmap(pixmap);
+	m_overlay->setPixmap(pixmap);
+
+	m_overlay->hide();
 
 	// Create overlay buttons
-	m_definitions_button = new QLabel(overlay);
+	m_definitions_button = new QLabel(m_overlay);
 	m_definitions_button->setPixmap(QIcon(":/definitions.png").pixmap(24,24));
 	m_definitions_button->setCursor(Qt::PointingHandCursor);
 	m_definitions_button->setToolTip(tr("Definitions"));
+	m_definitions_button->setDisabled(true);
 	m_definitions_button->installEventFilter(this);
 
-	m_hint_button = new QLabel(overlay);
+	m_hint_button = new QLabel(m_overlay);
 	m_hint_button->setPixmap(QIcon(":/hint.png").pixmap(24,24));
 	m_hint_button->setCursor(Qt::PointingHandCursor);
 	m_hint_button->setToolTip(tr("Hint"));
@@ -132,12 +123,12 @@ Window::Window() {
 	connect(m_board, &Board::hintAvailable, m_hint_button, &QLabel::setEnabled);
 
 	// Create clock
-	m_clock = new Clock(overlay);
+	m_clock = new Clock(m_overlay);
 	m_clock->setDisabled(true);
 	connect(m_clock, &Clock::togglePaused, m_board, &Board::togglePaused);
 	connect(m_board, &Board::loading, m_clock, &Clock::setLoading);
 
-	QHBoxLayout* overlay_layout = new QHBoxLayout(overlay);
+	QHBoxLayout* overlay_layout = new QHBoxLayout(m_overlay);
 	overlay_layout->setContentsMargins(0, 0, 0, 0);
 	overlay_layout->setSpacing(0);
 	overlay_layout->addSpacing(10);
@@ -154,33 +145,39 @@ Window::Window() {
 	layout->setSpacing(0);
 	layout->addWidget(view, 0, 0);
 	layout->addWidget(m_success, 0, 0, Qt::AlignCenter);
-	layout->addWidget(overlay, 0, 0, Qt::AlignHCenter | Qt::AlignTop);
+	layout->addWidget(m_overlay, 0, 0, Qt::AlignHCenter | Qt::AlignTop);
 
 	// Create menus
 	QMenu* menu = menuBar()->addMenu(tr("&Game"));
-	menu->addAction(tr("&New"), this, SLOT(newGame()), QKeySequence::New);
-	menu->addAction(tr("&Choose..."), this, SLOT(chooseGame()));
+	menu->addAction(tr("&New"), this, &Window::newGame, QKeySequence::New);
+	menu->addAction(tr("&Choose..."), this, &Window::chooseGame, QKeySequence::Open);
 	menu->addSeparator();
-	m_pause_action = menu->addAction(tr("&Pause"), m_board, SLOT(togglePaused()), tr("P"));
+	m_pause_action = menu->addAction(tr("&Pause"), m_board, &Board::setPaused, tr("P"));
+	m_pause_action->setCheckable(true);
 	m_pause_action->setDisabled(true);
-	QAction* action = menu->addAction(tr("&Hint"), m_board, SLOT(showHint()), tr("H"));
+	QAction* action = menu->addAction(tr("&Hint"), m_board, &Board::showHint, tr("H"));
 	action->setDisabled(true);
 	connect(m_board, &Board::hintAvailable, action, &QAction::setEnabled);
-	menu->addAction(tr("D&efinitions"), m_definitions, SLOT(selectWord()), tr("D"));
+	m_definitions_action = menu->addAction(tr("D&efinitions"), this, &Window::showDefinitions, tr("D"));
+	m_definitions_action->setDisabled(true);
 	menu->addSeparator();
-	menu->addAction(tr("&Details"), this, SLOT(showDetails()));
-	menu->addAction(tr("&Scores"), m_scores, SLOT(exec()));
+	m_details_action = menu->addAction(tr("&Details"), this, &Window::showDetails);
+	m_details_action->setDisabled(true);
+	menu->addAction(tr("High &Scores"), this, &Window::showScores, tr("Ctrl+H"));
 	menu->addSeparator();
-	action = menu->addAction(tr("&Quit"), qApp, SLOT(quit()), QKeySequence::Quit);
+	action = menu->addAction(tr("&Quit"), qApp, &QApplication::quit, QKeySequence::Quit);
 	action->setMenuRole(QAction::QuitRole);
 
 	menu = menuBar()->addMenu(tr("&Settings"));
-	menu->addAction(tr("Application &Language..."), this, SLOT(setLocale()));
+	QAction* dark_mode = menu->addAction(tr("&Dark Mode"), m_board, &Board::setDarkMode);
+	dark_mode->setCheckable(true);
+	menu->addSeparator();
+	menu->addAction(tr("Application &Language..."), this, &Window::setLocale);
 
 	menu = menuBar()->addMenu(tr("&Help"));
-	action = menu->addAction(tr("&About"), this, SLOT(about()));
+	action = menu->addAction(tr("&About"), this, &Window::about);
 	action->setMenuRole(QAction::AboutRole);
-	action = menu->addAction(tr("About &Qt"), qApp, SLOT(aboutQt()));
+	action = menu->addAction(tr("About &Qt"), qApp, &QApplication::aboutQt);
 	action->setMenuRole(QAction::AboutQtRole);
 
 	// Restore window geometry
@@ -188,32 +185,37 @@ Window::Window() {
 	resize(800, 600);
 	restoreGeometry(settings.value("Geometry").toByteArray());
 
+	// Set dark mode
+	if (settings.value("DarkMode", false).toBool()) {
+		dark_mode->trigger();
+	}
+
 	// Continue previous or start new game
 	show();
-	if (settings.contains("Current/Words") && (settings.value("Current/Version" ).toInt() == 4)) {
-		m_board->openGame();
-	} else {
-		settings.remove("Current");
+	if (!m_board->continueGame()) {
 		newGame();
 	}
 }
 
 //-----------------------------------------------------------------------------
 
-Window::~Window() {
+Window::~Window()
+{
 	m_board->saveGame();
 }
 
 //-----------------------------------------------------------------------------
 
-void Window::newGame() {
+void Window::newGame()
+{
 	NewGameDialog dialog(m_board, this);
 	dialog.exec();
 }
 
 //-----------------------------------------------------------------------------
 
-void Window::chooseGame() {
+void Window::chooseGame()
+{
 	bool ok = false;
 	QString number = QInputDialog::getText(this, tr("Choose Game"), tr("Game Number:"), QLineEdit::Normal, QString(), &ok);
 	if (ok && !number.isEmpty()) {
@@ -226,7 +228,8 @@ void Window::chooseGame() {
 
 //-----------------------------------------------------------------------------
 
-void Window::closeEvent(QCloseEvent* event) {
+void Window::closeEvent(QCloseEvent* event)
+{
 	m_board->saveGame();
 	QSettings().setValue("Geometry", saveGeometry());
 	QMainWindow::closeEvent(event);
@@ -234,10 +237,11 @@ void Window::closeEvent(QCloseEvent* event) {
 
 //-----------------------------------------------------------------------------
 
-bool Window::eventFilter(QObject* object, QEvent* event) {
+bool Window::eventFilter(QObject* object, QEvent* event)
+{
 	if (event->type() == QEvent::MouseButtonPress) {
 		if (object == m_definitions_button) {
-			m_definitions->selectWord();
+			showDefinitions();
 			return true;
 		} else if (object == m_hint_button) {
 			m_board->showHint();
@@ -249,20 +253,12 @@ bool Window::eventFilter(QObject* object, QEvent* event) {
 
 //-----------------------------------------------------------------------------
 
-bool Window::event(QEvent* event) {
-	if (event->type() == QEvent::WindowBlocked || event->type() == QEvent::WindowDeactivate) {
-		m_board->setPaused(true);
-	}
-	return QMainWindow::event(event);
-}
-
-//-----------------------------------------------------------------------------
-
-void Window::about() {
+void Window::about()
+{
 	QMessageBox::about(this, tr("About"), QString("<p><center><big><b>%1 %2</b></big><br/>%3<br/><small>%4<br/>%5</small></center></p><p><center>%6</center></p>")
 		.arg(tr("Connectagram"), QCoreApplication::applicationVersion(),
 			tr("A word unscrambling game"),
-			tr("Copyright &copy; 2009-%1 by Graeme Gott").arg("2020"),
+			tr("Copyright &copy; 2009-%1 by Graeme Gott").arg("2021"),
 			tr("Released under the <a href=\"http://www.gnu.org/licenses/gpl.html\">GPL 3</a> license"),
 			tr("Definitions are from <a href=\"http://wiktionary.org/\">Wiktionary</a>"))
 	);
@@ -270,55 +266,74 @@ void Window::about() {
 
 //-----------------------------------------------------------------------------
 
-void Window::showDetails() {
+void Window::showDefinitions()
+{
+	const int height = m_definitions_button->height();
+	const QPoint pos = m_definitions_button->mapToGlobal(QPoint(0, height));
+	m_definitions->popup(pos);
+}
+
+//-----------------------------------------------------------------------------
+
+void Window::showDetails()
+{
 	Pattern* pattern = m_board->pattern();
 	if (!pattern) {
 		return;
 	}
-	QString patternid = QSettings().value("Current/Pattern", 0).toString();
-	static const QStringList sizes = QStringList() << NewGameDialog::tr("Low")
-		<< NewGameDialog::tr("Medium")
-		<< NewGameDialog::tr("High")
-		<< NewGameDialog::tr("Very High");
-	QString number = "4"
-		+ m_board->words()->language()
-		+ patternid
-		+ QString::number(pattern->wordCount())
-		+ QString("%1").arg(int(pattern->wordLength() - 4), 2, 16, QLatin1Char('0'))
-		+ QString::number(pattern->seed(), 16);
 	QMessageBox dialog(QMessageBox::Information,
 		tr("Details"),
 		QString("<p><b>%1</b> %2<br><b>%3</b> %4<br><b>%5</b> %6<br><b>%7</b> %8<br><b>%9</b> %10</p>")
-			.arg(tr("Pattern:")).arg(pattern->name())
-			.arg(NewGameDialog::tr("Language:")).arg(LocaleDialog::languageName(m_board->words()->language()))
-			.arg(NewGameDialog::tr("Amount of Words:")).arg(sizes.value(pattern->wordCount()))
-			.arg(NewGameDialog::tr("Word Length:")).arg(NewGameDialog::tr("%n letter(s)", "", pattern->wordLength() + 1))
-			.arg(tr("Game Number:")).arg(number),
+			.arg(tr("Pattern:")
+				, pattern->name()
+				, NewGameDialog::tr("Language:")
+				, WordList::languageName(m_board->words()->language())
+				, NewGameDialog::tr("Amount of Words:")
+				, NewGameDialog::densityString(pattern->wordCount())
+				, NewGameDialog::tr("Word Length:")
+				, NewGameDialog::tr("%n letter(s)", "", pattern->wordLength() + 1)
+				, tr("Game Number:"))
+			.arg(m_board->gameNumber()),
 		QMessageBox::NoButton,
 		this);
-	dialog.setIconPixmap(QIcon(QString(":/patterns/%1.png").arg(patternid)).pixmap(96,96));
+	dialog.setIconPixmap(QIcon(QString(":/patterns/%1.png").arg(pattern->type())).pixmap(96,96));
 	dialog.exec();
 }
 
 //-----------------------------------------------------------------------------
 
-void Window::setLocale() {
+void Window::showScores()
+{
+	ScoresDialog scores(this);
+	scores.exec();
+}
+
+//-----------------------------------------------------------------------------
+
+void Window::setLocale()
+{
 	LocaleDialog dialog(this);
 	dialog.exec();
 }
 
 //-----------------------------------------------------------------------------
 
-void Window::gameStarted() {
+void Window::gameStarted()
+{
+	m_overlay->show();
 	m_clock->setEnabled(true);
 	m_clock->start();
 	m_pause_action->setEnabled(true);
+	m_definitions_button->setEnabled(true);
+	m_definitions_action->setEnabled(true);
+	m_details_action->setEnabled(true);
 	m_success->hide();
 }
 
 //-----------------------------------------------------------------------------
 
-void Window::gameFinished() {
+void Window::gameFinished()
+{
 	QSettings settings;
 	int count = settings.value("Current/Count").toInt();
 	int length = settings.value("Current/Length").toInt();
@@ -329,13 +344,19 @@ void Window::gameFinished() {
 	m_pause_action->setDisabled(true);
 	m_success->show();
 
-	m_scores->addScore(std::lround(msecs / 1000.0), count, length);
+	ScoresDialog scores(this);
+	if (scores.addScore(std::lround(msecs / 1000.0), count, length)) {
+		scores.exec();
+	}
 }
 
 //-----------------------------------------------------------------------------
 
-void Window::gamePauseChanged() {
+void Window::gamePauseChanged()
+{
 	bool paused = m_board->isPaused();
-	m_pause_action->setText(paused ? tr("&Resume") : tr("&Pause"));
+	m_pause_action->setChecked(paused);
 	m_clock->setPaused(paused);
 }
+
+//-----------------------------------------------------------------------------
